﻿using System.Collections.ObjectModel;
using System.Linq;
using Sample.Models;
using System.ComponentModel;

namespace Sample.MenuView
{
    public class MenuViewPresenter : IMenuViewPresenter, INotifyPropertyChanged
    {
        private readonly ObservableCollection<ItemGroup> _ItemGroups;

        public MenuViewPresenter(IMenuView view)
        {
            _ItemGroups = new ObservableCollection<ItemGroup>();

            View = view;
            View.Presenter = this;
        }

        public IMenuView View { get; private set; }

        public ObservableCollection<ItemGroup> ItemGroups
        {
            get
            {
                return _ItemGroups;
            }
        }

        public bool AddMenuItem(string menuGroup, string itemName)
        {
            var item = AddItem(menuGroup, itemName);

            if (item == null)
            {
                return false;
            }

            return true;
        }

        public bool AddMenuItem(string menuGroup, string itemName, bool selected)
        {
            var item = AddItem(menuGroup, itemName);

            if (item == null)
            {
                return false;
            }

            if (selected)
                SelectView(itemName);

            return true;
        }

        private Item AddItem(string menuGroup, string itemName)
        {
            var group = ItemGroups.FirstOrDefault(ig => ig.Title == menuGroup);

            if (group == null)
            {
                group = new ItemGroup(menuGroup);
                ItemGroups.Add(group);
            }

            var item = group.Items.FirstOrDefault(i => i.Text == itemName);

            if (item == null)
            {
                item = new Item(itemName, new Item.SelectViewDelegate(OnSelectView));
                group.Items.Add(item);
            }

            OnPropertyChanged("ItemGroups");

            return item;
        }

        public bool SelectView(string viewName)
        {
            if (string.IsNullOrWhiteSpace(viewName))
            {
                return false;
            }

            foreach (var group in _ItemGroups)
            {
                foreach (var item in group.Items)
                {
                    if (item.Text == viewName)
                    {
                        group.SelectedItem = item;
                        return true;
                    }
                }
            }

            return false;
        }

        private void OnSelectView(Item seletedItem)
        {
            if (seletedItem == null)
            {
                return;
            }

            string groupName = (from ig in _ItemGroups
                                where ig.Items.Contains(seletedItem)
                                select ig.Title).FirstOrDefault();

            if (!string.IsNullOrWhiteSpace(groupName))
            {
                View.SelectGroup(groupName);
            }
        }

        #region INotifyPropertyChanged Members

        private void OnPropertyChanged(string prop)
        {

            if (PropertyChanged != null)
            {
                PropertyChanged(this, new System.ComponentModel.PropertyChangedEventArgs(prop));
            }
        }

        public event System.ComponentModel.PropertyChangedEventHandler PropertyChanged;

        #endregion
    }
}